import shelve
from shelve import Shelf
import os
import copy
class Archiver:
    def __init__(self):
        self.home = os.path.expanduser("~")
        self.path: str = os.path.join(self.home, ".story", "db", "archive")

    def addStory(self, title: str, bookObj : dict):
        os.makedirs(os.path.dirname(self.path), exist_ok=True)
        db: Shelf = shelve.open(self.path)
        if title in db.keys():
            copyDict : dict = copy.deepcopy(db[title])
            copyDict.update(bookObj)
            db[title] = copyDict
        else:
            db[title] = bookObj
        db.close()

    def getStory(self, title: str) -> dict:
        db: Shelf = shelve.open(self.path)
        bookObj = copy.deepcopy(db[title])
        db.close()
        return bookObj



import argparse
import story_analyzer.get_book as get_book
from story_analyzer.book import Book
import re
import colorama
from colorama import Fore, Style
from story_analyzer.archiver import Archiver
import json


def panic(message):
    print(Fore.RED + f"error: {message}")
    reset = Style.RESET_ALL
    exit(1)

def main():
    args_parser = argparse.ArgumentParser(
        description='Book Analyzer: get insight informations of your storie')
    args_parser.add_argument('input', nargs='?', type=str, metavar='input',
                             help='input file path or book name (only in web mode)')
    args_parser.add_argument('output', nargs=1, type=str,
                             metavar='output', help='output file path. The extension automatic is set to JSON')
    args_parser.add_argument('-m', '--mode', nargs=1, type=str,
                             choices=['local', 'web'], help='app modes', default=['local'])
    args_parser.add_argument(
        '-q', '--quiz', action='store_true', help='quiz game')
    args_parser.add_argument('-t', '--translate', nargs='?', type=str, choices=[
                             'French', 'German', 'Romanian'], help='translate the book', const=['French'])
    args_parser.add_argument(
        '-d', '--discussions', action='store_true', help='list the book discussions (topics)')
    args_parser.add_argument(
        '-s', '--summary', action='store_true', help='summarize the book')
    args_parser.add_argument(
        '-l', '--language', action='store_true', help='detect book language')
    args_parser.add_argument('-c', '--characters', action='store_true',
                             help='get informations of the book characters')
    args_parser.add_argument('-a', '--actions', nargs='?', type=int,
                             help='get the top most actions of a book', const=10)
    # TODO add information about which queries will be saved
    args_parser.add_argument('--save', nargs=1, type=str,
                             help='the book will be saved and so will any queries invoked deemed savable.', metavar='title')
    args_parser.add_argument('--read', nargs=1, type=str,
                             help='for queries saved the response time will be quicker, since the query done '
                                  'will be a lookup to a database with the book info. However if a query was not saved or '
                                  'is not deemed savable then the query times will remain the same', metavar='title')
    args_parser.add_argument('-p', '--projection', nargs=1, type=str,
                             help='project queries in the text range. Projetion type is [<bottom>;<higher>]')
    args_parser.add_argument('-sa', '--sentiment_analysis', action='store_true',
                             help="sentiment analysis of the book")
    args_parser.add_argument('-v', '--view', action='store_true',
                             help="view book content. With projection is used, the text is limited by the projection range")
    args_parser.add_argument('-si', '--similar', nargs=1, type=str,
                             help='finds the sentence that better describes the input given. It also gives the word offset'
                                  ' from the beginning of the story. Useful for finding an exact reference.')
    args_parser.add_argument('-sn', '--sentence_no', action='store_true', help='get the number of sentences of a story')


    args = args_parser.parse_args()

    try:
        if args.input:
            book_content = get_book.get_book(args.mode[0], args.input)
            # limit book_content with projection is used
            book = Book(book_content)
            if args.projection and args.input and not args.read:

                pattern = r'\[(\d+):(\d+)\]'
                match = re.search(pattern, str(args.projection[0]))

                if match:
                    bottom_limit = int(match.group(1))
                    higher_limit = int(match.group(2))

                    try:
                        book.setProjection(bottom_limit, higher_limit)
                    except TypeError as e:
                        panic(str(e))
        elif args.read:
            archive = Archiver()
            bookObj = archive.getStory(args.read[0])
            if bookObj and "content" in bookObj.keys():
                book_content = bookObj["content"]
            else:
                panic("There wasn't a save for this title before this reading command")
            book = Book(title=args.read[0])

        out = {}
        saveDict = {}

        if args.sentence_no:
            if args.read and "sentence_no" in bookObj.keys():
                sentence_no = bookObj["sentence_no"]
            else:
                sentence_no = book.spacy_queries.querySentences()
            out["sentence_no"] = sentence_no
            if args.save:
                saveDict["sentence_no"] = sentence_no
        if args.similar:
            _input = args.similar[0]
            similar = book.spacy_queries.similarSentence(_input)
            out["similar"] = similar
        if args.actions:
            # if it's requested a read and the book info is cached then use it else do the query
            if args.read and "actions" in bookObj.keys():
                actions = bookObj["actions"]
            else:
                actions = book.spacy_queries.queryActions(args.actions)
            out["actions"] = actions
            if args.save:
                saveDict["actions"] = actions
        if args.language:
            if args.read and "language" in bookObj.keys():
                language = bookObj["language"]
            else:
                language = book.queryLanguage()
            out["language"] = language
            if args.save:
                saveDict["language"] = language
        if args.quiz:
            if args.read and "sentences" in bookObj.keys():
                quiz_sentences = bookObj["sentences"]
            else:
                quiz_sentences = book.quiz()
            out["sentences"] = quiz_sentences
            if args.save:
                saveDict["sentences"] = quiz_sentences
        if args.translate:
            if args.read and "translation" in bookObj.keys() and (args.translate in bookObj["translation"].keys()):
                translation = bookObj["translation"][args.translate]
            else:
                try:
                    translation = book.translate(args.translate)
                except TypeError as e:
                    panic(str(e))
                
            out["translation"] = translation
            if args.save:
                saveDict["translation"] = {args.translate: translation}
        if args.summary:
            if args.read and "summary" in bookObj.keys():
                summary = bookObj["summary"]
            else:
                summary = book.summarize()
            out["summary"] = summary
            if args.save:
                saveDict["summary"] = summary
        if args.discussions:
            topics = book.topics()
            print(topics)
        if args.characters:
            if args.read and "characters" in bookObj.keys():
                charactersInfo = bookObj["characters"]
            else:
                charactersInfo = book.spacy_queries.getCharacters()
            out["characters"] = charactersInfo
            if args.save:
                saveDict["characters"] = charactersInfo
        if args.sentiment_analysis:
            if args.read and "sentiment" in bookObj.keys():
                sentiment = bookObj["sentiment"]
            else:
                sentiment = book.sentiment()
            out["sentiment"] = sentiment
            if args.save:
                saveDict["sentiment"] = sentiment
            print(book.sentiment())
        if args.view:
            out["view"] = book.content

        with open(args.output[0] + ".json", 'w') as file:
            json.dump(out, file, indent=4)

        if args.save:
            book.saveContent(args.save[0], saveDict)

    except ValueError as e:
        panic(str(e))


if __name__ == "__main__":
    main()



import os

import spacy
from spacy.cli import download

class ParserModels:
    def __init__(self):
        self.path: str = os.path.dirname(__file__) + "/data/spacy_models"
        self.file = open(self.path, "r", encoding="utf-8")
        self.languages: dict = {}
        for line in self.file.readlines():
            tokens = line.split()
            self.languages[tokens[0]] = tokens[1]

    def getModel(self, language : str) -> str:
        if language not in self.languages.keys():
            return None
        try:
            spacy.load(self.languages[language])
        except:
            download(self.languages[language])

        return self.languages[language]



import spacy
from collections import Counter
from typing import List,Tuple
import random
from story_analyzer.parser_models import ParserModels
from sortedcontainers.sortedset import SortedSet
from story_analyzer.archiver import Archiver
from typing import Optional

class SpacyQueries:
    # The goal of this class is to encapsulate all the spacy queries that are
    # done with the same text and same model
    def __init__(self, language : Optional[str]=None, text : Optional[str] = None, title : Optional[str] = None):
        self.parser: ParserModels = ParserModels()
        if text and language:
            self.model:str = self.__modelFromLanguage(language)
            self.text = text
            self.docReady = False
        elif title:
            archive = Archiver()
            bookObj: dict = archive.getStory(title)
            self.nlp = bookObj["nlp"]
            self.doc = bookObj["doc"]
            # If there is a read than it is guaranteed that there is a language saved
            self.model:str = self.__modelFromLanguage(bookObj["language_abr"])
            self.docReady = True

    def __createDoc(self):
        self.nlp = spacy.load(self.model)
        self.doc = self.nlp(self.text)
        self.docReady = True

    def __modelFromLanguage(self, language: str) -> str :
        return self.parser.getModel(language)

    def saveDoc(self, title: str):
        self.__createOne()
        archive = Archiver()
        archive.addStory(title, {"doc": self.doc, "nlp": self.nlp})

    def __createOne(self):
        if not self.docReady:
            self.__createDoc()

    def getStopWords(self):
        self.__createOne()
        return self.nlp.Defaults.stop_words

    def queryActions(self, top : int):
        "Gets the most actions of a book"
        # Acts as a singleton
        self.__createOne()
        verbs: Counter = Counter()
        for w in self.doc:
            if w.pos_ == "VERB":
                verbs[w.lemma_] += 1
        return verbs.most_common(top)

    def querySentences(self):
        "Gets the number of sentences of a text"
        # Acts as a singleton
        self.__createOne()
        return len(list(self.doc.sents))

    def queryGetRandomSentence(self):
        "Gets a random sentence of a text. The random sentence must have more than 20 tokens"
        self.__createOne()
        random_sentence = random.choice(list(self.doc.sents))
        while len(random_sentence) < 20:
            random_sentence = random.choice(list(self.doc.sents))
        return random_sentence

    def querySentencesInRange(self,bottom,higher):
        "Gets a the sentence number 'bottom' until sentence number 'higher'. Sentences is concatened in a single sting"
        
        self.__createOne()

        if bottom < 0 and higher > len(list(self.doc.sents)):
            raise TypeError("projection not allowed")

        sentences = (list(self.doc.sents))[bottom:higher]
        sentences_string = "".join(str(sentences))

        return sentences_string

    def similarSentence(self, input : str) -> List[Tuple[str, int]]:
        "Given a summary of a sentence it returns the actual sentence and the offset where it is present"
        # Acts as a singleton
        self.__createOne()
        #requires that the language of the input is the same
        search_sent = self.nlp(input)

        search_no_stop_words = self.nlp(' '.join([str(word) for word in search_sent if not word.is_stop]))

        simSet = SortedSet(key=lambda x: -x[1])

        for sent in self.doc.sents:
            sent_no_stop_words = self.nlp(' '.join([str(word) for word in sent if not word.is_stop]))
            entry = (sent.text, search_no_stop_words.similarity(sent_no_stop_words), sent.start)
            simSet.add(entry)

        # for now hardcoded if there's time enable custom number
        return list(simSet[0:2])

    def getCharacters(self):
        "Get characters information of a book"
        
        # Acts as a singleton
        self.__createOne()
        
        characters = {}

        for sent in self.doc.sents:
            for ent in sent.ents:
                if ent.label_ == "PERSON":
                    name = ent.text.lower().strip()
                    characters[name] = characters.get(name, 0) + 1
        
        return characters


import requests
import re
from bs4 import BeautifulSoup, NavigableString


def decompose_tag(tags):
    for tag in tags:
        tag.decompose()


def remove_useless_tags(soup):
    decompose_tag(soup.find_all('head'))
    decompose_tag(soup.find_all('div', {'class': 'footnote'}))
    decompose_tag(soup.find_all('div', {'class': 'footnotes'}))
    decompose_tag(soup.find_all('hr'))
    decompose_tag(soup.find_all('span', {'class': 'pagenum'}))
    decompose_tag(soup.find_all('div', {'class': 'caption'}))
    decompose_tag(soup.find_all('span', {'class': 'caption'}))
    decompose_tag(soup.find_all('div', {'class': 'toc'}))
    decompose_tag(soup.find_all('p', {'class': 'toc'}))
    decompose_tag(soup.find_all('div', {'class': 'blk'}))
    decompose_tag(soup.find_all('h1'))
    decompose_tag(soup.find_all('section'))
    decompose_tag(soup.find_all('div', {'class': 'dedication'}))
    decompose_tag(soup.find_all('div', {'class': 'epigraph'}))
    decompose_tag(soup.find_all('h2'))
    decompose_tag(soup.find_all('div', {'class': 'figcenter'}))
    decompose_tag(soup.find_all('a'))
    decompose_tag(soup.find_all('img'))
    decompose_tag(soup.find_all('p', {'class': 'c'}))
    decompose_tag(soup.find_all('table'))
    decompose_tag(soup.find_all('div', style=True))
    return soup


def treat_html(html):
    result = ""
    soup = remove_useless_tags(BeautifulSoup(html, 'html.parser'))
    nested = []
    for tag in soup.find_all():
        if tag in nested:
            nested.remove(tag)
        elif isinstance(tag, NavigableString):
            result += tag
        else:
            nested_tags = [
                t for t in tag.descendants if t.name is not None]
            for x in nested_tags:
                nested.append(x)
            if tag not in nested:
                result += tag.get_text()
    return result


def trim_newlines(text):
    return re.sub(r'\s*\n\s*', '\n', text)


def get_book(mode, book_name):
    book_string = ''

    if mode == 'local':
        input_file = open(book_name, 'r')
        book_string = input_file.read()
        input_file.close()
    elif mode == 'web':
        search_url = f"http://www.gutenberg.org/ebooks/search/?query={book_name}"
        response = requests.get(search_url)

        if response.status_code == 200:
            soup = BeautifulSoup(response.content, 'html.parser')
            book_id = soup.find('li', {'class': 'booklink'}).find('a')[
                'href'].split('/')[-1]
            book_html_url = f"http://www.gutenberg.org/cache/epub/{book_id}/pg{book_id}-images.html"
            response = requests.get(book_html_url)
            if response.status_code == 200:
                book_string = trim_newlines(treat_html(response.content))
            else:
                raise ValueError(f"unavailable html version of the book")
        else:
            raise ValueError(f"unavailable book: {book_name}")

    return book_string




from transformers import T5Tokenizer, T5ForConditionalGeneration, GPT2Tokenizer, GPT2LMHeadModel, PegasusForConditionalGeneration, PegasusTokenizer, pipeline, AutoModelForSequenceClassification, AutoTokenizer, logging
from story_analyzer.spacy_queries import SpacyQueries
import torch
import random
import gensim
from story_analyzer.archiver import Archiver
from typing import Optional
from story_analyzer.parser_models import ParserModels
import spacy

class Book:

    def __init__(self, content: Optional[str]=None, title: Optional[str] = None) -> None:
        logging.set_verbosity_error()
        self.db : Archiver = Archiver()
        if content:
            self.content: str = content
            self.read_only: bool = False
            language_abr, language_long = self.__detectLanguage()
            self.language: str = language_long
            self.language_abr = language_abr
            self.spacy_queries: SpacyQueries = SpacyQueries(
                language_abr, self.content)
        elif title:
            bookObj: dict = self.db.getStory(title)
            # These fields are always stored when a save is done
            if bookObj["content"] and bookObj["language"]:
                self.content = bookObj["content"]
                self.language = bookObj["language"]
                self.language_abr = bookObj["language_abr"]
            else:
                raise Exception('A save was not previously done')

            self.spacy_queries: SpacyQueries = SpacyQueries(title=title)
            self.read_only: bool = True

        self.device = 0 if torch.cuda.is_available() else -1

    def quiz(self):
        "Generate a quiz game with six false sentences and one true sentence. The goal is the user guess the true sentence"

        # get a random true sentence from the text
        random_sentence = self.spacy_queries.queryGetRandomSentence()

        # load GPT model and tokenizer
        gpt_model = GPT2LMHeadModel.from_pretrained('gpt2')
        gpt_tokenizer = GPT2Tokenizer.from_pretrained('gpt2')

        # encode the first 5 words from the sentence to put in the model
        content_ids = gpt_tokenizer.encode(
            str(random_sentence[0:5]), return_tensors='pt')

        # generate six fake sentences
        generated_text_samples = gpt_model.generate(
            content_ids,
            max_length=50,
            do_sample=True,
            top_k=100,
            top_p=0.92,
            temperature=0.8,
            repetition_penalty=1.5,
            num_return_sequences=6
        )

        # decode fake sentences
        fake_sentences = []
        for beam in generated_text_samples:
            fake_sentences.append(str(gpt_tokenizer.decode(
                beam, skip_special_tokens=True)))

        # put the true sentence in a random position
        random_index = random.randint(0, len(fake_sentences))
        fake_sentences.insert(random_index, str(random_sentence))

        return fake_sentences

    def translate(self, toLanguage="Germany"):
        "translate the book content"

        if len(self.content) > 512:
            raise TypeError("The text are to long to be translated. Try a little one.")

        # load t5 model and tokenizer
        t5_model = T5ForConditionalGeneration.from_pretrained(
            't5-small', return_dict=True)
        t5_tokenizer = T5Tokenizer.from_pretrained(
            't5-small', model_max_length=10000)

        # encode text to put in the model
        input_ids = t5_tokenizer("translate " + self.language + "to " +
                                 toLanguage + ": " + self.content, return_tensors="pt").input_ids

        # generate the translated content
        outputs_ids = t5_model.generate(
            input_ids, max_length=10000, num_beams=4)

        # decode translated text
        translation = t5_tokenizer.decode(
            outputs_ids[0], skip_special_tokens=True)

        return translation

    def summarize(self):
        "summarize the book content"

        if len(self.content) > 1024:
            raise TypeError("The text are to long to be translated. Try a little one.")

        # load Pegasus model and tokenizer
        pegasus_model = PegasusForConditionalGeneration.from_pretrained(
            'google/pegasus-cnn_dailymail')
        pegasus_tokenizer = PegasusTokenizer.from_pretrained(
            'google/pegasus-cnn_dailymail')

        # encode text to put in the model
        input_ids = pegasus_tokenizer(
            self.content, truncation=True, padding="longest", return_tensors='pt')

        # generate the summary content
        output_ids = pegasus_model.generate(**input_ids)

        # decode summary text
        summary = pegasus_tokenizer.batch_decode(output_ids[0])

        return ' '.join(summary)

    def topics(self):
        "get most relevant discussions from the book"

        # get stop words from space_queries
        stop_words = self.spacy_queries.getStopWords()

        data_words = gensim.utils.simple_preprocess(self.content, deacc=True)
        bigram = gensim.models.Phrases(data_words, min_count=5, threshold=100)
        bigram_mod = gensim.models.phrases.Phraser(bigram)

        def remove_stopwords(texts):
            return [[word for word in gensim.utils.simple_preprocess(str(doc)) if word not in stop_words] for doc in
                    texts]

        def make_bigrams(texts):
            return [bigram_mod[doc] for doc in texts]

        def lemmatization(texts, allowed_postags=['NOUN', 'ADJ', 'VERB', 'ADV']):
            texts_out = []
            for sent in texts:
                parser = ParserModels()
                model = parser.getModel(self.language_abr)
                nlp = spacy.load(model)
                doc = nlp(" ".join(sent))
                texts_out.append(
                    [token.lemma_ for token in doc if token.pos_ in allowed_postags])
            return texts_out

        data_words_nostops = remove_stopwords(data_words)
        data_words_bigrams = make_bigrams(data_words_nostops)
        data_lemmatized = lemmatization(data_words_bigrams, allowed_postags=[
                                        'NOUN', 'ADJ', 'VERB', 'ADV'])

        id2word = gensim.corpora.Dictionary(data_lemmatized)
        texts = data_lemmatized
        corpus = [id2word.doc2bow(text) for text in texts]

        lda_model = gensim.models.ldamodel.LdaModel(
            corpus=corpus,
            id2word=id2word,
            num_topics=15,
            random_state=100,
            update_every=1,
            chunksize=100,
            passes=10,
            alpha='auto',
            per_word_topics=True
        )

        return lda_model.print_topics()

    def saveContent(self, title : str, archiveDict : dict):
        #works because self.language is not used to init spacy queries and nor is the archive used for that
        archiveDict.update({"content": self.content, "language": self.language, "language_abr": self.language_abr})
        self.db.addStory(title, archiveDict)
        self.spacy_queries.saveDoc(title)

    def getContent(self, title : str):
        return self.db.getStory(title)

    def sentiment(self) -> str:
        model = AutoModelForSequenceClassification.from_pretrained(
            "bert-base-multilingual-uncased")
        tokenizer = AutoTokenizer.from_pretrained(
            "bert-base-multilingual-uncased")
        pipe = pipeline("sentiment-analysis", model=model,
                        tokenizer=tokenizer, device=self.device)

        chunks = self.content.split(".")

        labels = {"Neutral": 0, "Positive": 0, "Negative": 0}

        dataset = [{"text": text} for text in chunks]

        results = pipe(dataset, batch_size=8)

        for result in results:
            label = result["label"]
            if label == "LABEL_0":
                labels["Negative"] += 1
            elif label == "LABEL_1":
                labels["Positive"] += 1
            else:
                labels["Neutral"] += 1

        return max(labels, key=labels.get)

    def __detectLanguage(self) -> str:
        model_ckpt = "papluca/xlm-roberta-base-language-detection"
        pipe = pipeline("text-classification", model=model_ckpt)
        preds = pipe(self.content, top_k=None, truncation=True, max_length=128)
        if preds:
            pred = preds[0]
            # if this method is called then it stores in a instance variable
            language_long = ''
            if (pred["label"] == "es"):
                language_long = "Spanish"
            elif (pred["label"] == "en"):
                language_long = "English"
            elif (pred["label"] == "de"):
                language_long = "German"
            elif (pred["label"] == "it"):
                language_long = "Italian"
            elif (pred["label"] == "pt"):
                language_long = "Portuguese"
            elif (pred["label"] == "fr"):
                language_long = "French"

            return pred["label"], language_long
        else:
            return None

    def queryLanguage(self) -> str:
        return self.language

    def setProjection(self,bottom,higher):
        self.content = self.spacy_queries.querySentencesInRange(bottom=bottom,higher=higher)


from unittest import TestCase
from story_analyzer.archiver import Archiver

class TestArchiver(TestCase):
    def test_add_story(self):
        db = Archiver()
        harry = {"Harry Potter 1": {"title" : "Harry Potter 1", "text" : "You know who!"}}
        db.addStory("Harry Potter 1", harry)
        self.assertEqual(harry, db.getStory("Harry Potter 1"))
        self.assertNotEqual({}, db.getStory("Harry Potter 1"))



from unittest import TestCase
from story_analyzer.spacy_queries import SpacyQueries

class TestSpacyQueries(TestCase):
    def test_similar_sentence(self):
        queries = SpacyQueries("pt","O harry potter está com a Hermione a estudar. O Harry Potter está a jantar com o Ron.")
        self.assertEqual("O Harry Potter está a jantar com o Ron.", queries.similarSentence("O Harry está a comer.")[0])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['story_analyzer', 'story_analyzer.tests']

package_data = \
{'': ['*'], 'story_analyzer': ['data/*']}

install_requires = \
['beautifulsoup4>=4.12.2,<5.0.0',
 'colorama>=0.4.6,<0.5.0',
 'gensim>=4.3.1,<5.0.0',
 'sentencepiece>=0.1.97,<0.2.0',
 'sortedcontainers>=2.4.0,<3.0.0',
 'spacy>=3.5.1,<4.0.0',
 'torch>=2.0.0,<3.0.0',
 'transformers>=4.27.4,<5.0.0']

entry_points = \
{'console_scripts': ['story_analyzer = story_analyzer.main:main']}

setup_kwargs = {
    'name': 'story-analyzer',
    'version': '0.1.3',
    'description': '',
    'long_description': '# Story Analyzer\n\nGet all the details of a story. From summaries to character information, explore your book like never before!\n\n![Banner](banner.jpg)\n\n## Features\n\n* Quiz game\n* Translation \n* Summarization  \n* Most relevant topics\n* Characteres list \n* Lines list\n* Number of sentences\n* Match of input phrase with a storie excert\n\n## Usage\n\n```\nusage: main.py [-h] [-m {local,web}] [-q] [-t [{English,Spanish,French,German,Italian,Portuguese}]] [-d] [-s] [-l] [-c] [-a [ACTIONS]]\n               [--save title] [-p PROJECTION]\n               input output\n\nBook Analyzer: get insight informations of your storie\n\npositional arguments:\n  input                 input file path or book name (only in web mode)\n  output                output file path\n\noptions:\n  -h, --help            show this help message and exit\n  -m {local,web}, --mode {local,web}\n                        app modes\n  -q, --quiz            quiz game\n  -t [{English,Spanish,French,German,Italian,Portuguese}], --translate [{English,Spanish,French,German,Italian,Portuguese}]\n                        translate the book\n  -d, --discussions     list the book discussions (topics)\n  -s, --summary         summarize the book\n  -l, --language        detect book language\n  -c, --characters      get informations of the book characters\n  -a [ACTIONS], --actions [ACTIONS]\n                        get the top most actions of a book\n  --save title          the book will be saved and so will any queries invoked deemed savable.\n  -p PROJECTION, --projection PROJECTION\n                        project queries in the text range. Projection type is [<bottom>;<higher>]\n  -sa, --sentiment-analysis\n                        get the sentiment analysis of the book\n```\n\n## Dependencies\n\n* Quiz game is made with https://huggingface.co/gpt2\n* Transaltion is made with https://huggingface.co/docs/transformers/main/en/model_doc/t5#overview\n* Summarization is made with https://huggingface.co/docs/transformers/model_doc/pegasus\n* Most Relevent Topics is made with https://radimrehurek.com/gensim/\n* Language detection is made with https://huggingface.co/papluca/xlm-roberta-base-language-detection\n* Actions, Characters and places list is made with https://spacy.io/\n* HTML extraction is made with https://www.crummy.com/software/BeautifulSoup/bs4/doc/',
    'author': 'Francisco Alexandre Neves',
    'author_email': 'pg50375@alunos.uminho.pt',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
